/**
  @file
  @brief
  Datalogger Example 

  This example program demonstrates use of the datalogger apis. A 4KB sector is
  allocated as the datalog partition. One sector has 16 pages and page 0 is
  reserved by the lib. Pages 1 to 15 are available for use by the user. Once a
  page is program-ed, that page may not be overwritten. Pages may not be erased
  individually and the entire sector has to be erased. Therefore, users shall 
  ensure that pages are completely filled before programming into the datalogger.
  APIs are provided to read, program and erase. Page management is left to the 
  user application requirements. 

*/
/*
 * ============================================================================
 * History
 * =======
 * 10 Jan 2016 : Created
 *
 * (C) Copyright 2016, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#include <ft900.h>
#include <ft900_dlog.h>
/* support for printf output */
#include "../tinyprintf/tinyprintf.h"

/** Function used to facilitate printf */
void myputc(void* p, char c)
{
    uart_write(p, (uint8_t)c);
}

void setup(void)
{
    /* Enable the UART Device... */
    sys_enable(sys_device_uart0);
    /* Make GPIO48 function as UART0_TXD and GPIO49 function as UART0_RXD... */
    gpio_function(48, pad_uart0_txd); /* UART0 TXD */
    gpio_function(49, pad_uart0_rxd); /* UART0 RXD */
    uart_open(UART0,                    /* Device */
              1,                        /* Prescaler = 1 */
              UART_DIVIDER_19200_BAUD,  /* Divider = 1302 */
              uart_data_bits_8,         /* No. Data Bits */
              uart_parity_none,         /* Parity */
              uart_stop_bits_1);        /* No. Stop Bits */

    /* Print out a welcome message... */
    uart_puts(UART0,
        "\x1B[2J" /* ANSI/VT100 - Clear the Screen */
        "\x1B[H"  /* ANSI/VT100 - Move Cursor to Home */
            "(C) Copyright 2016, Future Technology Devices International Ltd. \r\n"
            "--------------------------------------------------------------------- \r\n"
            "Welcome to the Datalogger Example ...                                 \r\n"
        	"This example will erase the datalogger partition and fill all pages   \r\n"
        	"from 1 to 14 (14 pages) with repeated values of 0x00 to 0x0D after it \r\n"
           	"will erase the sector again and end.                                  \r\n"
            "--------------------------------------------------------------------- \r\n"
        );

    /* Enable tfp_printf() functionality... */
    init_printf(UART0,myputc);
}

extern __flash__ uint32_t __dlog_partition[];	/**< global variable indicating start of dlog partition in flash */
	
uint8_t	wrbuf[256] __attribute__((aligned(4)));
uint8_t	rdbuf[256] __attribute__((aligned(4)));

int cmp (uint8_t *a, uint8_t *b, int len)
{
	int i;

	for (i = 0; i < len; i++)
	{
		if (*a != *b)
		{
			tfp_printf ("cmp: mismatch a[%d]=0x%02x, b[%d]=0x%02x\n", i, *a, i, *b);
			return -1;
		}
		a++, b++;
	}
	return 0;
}

void dump (uint8_t *d, int len)
{
	int	i;

	for (i = 0; i < len; i++)
	{
		if ((i & 0xF) == 0)
			tfp_printf ("\n0x%02x: ", i);
		tfp_printf ("%02x ", *d++);
	}
	tfp_printf ("\n");
}

int main(void)
{
	int	ret;
	int	pgsz;
	int	pages;

	int i, j;

	setup();

	i = (int)__dlog_partition;

	tfp_printf ("__dlog_partition: %08X\n",i);

	ret = dlog_init (__dlog_partition, &pgsz, &pages);

	if (ret < 0)
	{
		tfp_printf ("dlog_init: failed\n");
		return 0;
	}

	tfp_printf ("dlog_init: passed, pgsz=0x%x, pages=%d\n", pgsz, pages);

	// erase the partition
	dlog_erase ();

	for (i = 0; i < pages; i++)
	{
		for (j = 0; j < pgsz; j++)
			wrbuf[j] = i;

		dlog_prog (i, (uint32_t *)wrbuf);
		
		for (j = 0; j < pgsz; j++)
			rdbuf[j] = 0;
		dlog_read (i, (uint32_t *)rdbuf);
		cmp (wrbuf, rdbuf, pgsz);
		dump (rdbuf, pgsz);
	}

	dlog_erase ();
	tfp_printf ("program ended\n");
	while (1);
	return 0;
}

